'use strict';
var lodash_1 = require('lodash');
var inflection_1 = require('inflection');
var jsonapi_serializer_1 = require('jsonapi-serializer');
var utils_1 = require('./utils');
/**
 * Mapper class for Bookshelf sources
 */
var Bookshelf = (function () {
    /**
     * Standard constructor
     */
    function Bookshelf(baseUrl, serialOpts) {
        this.baseUrl = baseUrl;
        this.serialOpts = serialOpts;
    }
    /**
     * Maps bookshelf data to a JSON-API 1.0 compliant object
     *
     * The `any` type data source is set for typing compatibility, but must be removed if possible
     * TODO fix data any type
     */
    Bookshelf.prototype.map = function (data, type, bookOpts) {
        if (bookOpts === void 0) { bookOpts = {}; }
        var linkOpts = { baseUrl: this.baseUrl, type: type, pag: bookOpts.pagination };
        // Set default values for the options
        lodash_1.defaultsDeep(bookOpts, {
            omitAttrs: [],
            keyForAttr: lodash_1.identity,
            relations: true,
            typeForModel: function (attr) { return inflection_1.pluralize(attr); },
            enableLinks: true
        });
        var info = { bookOpts: bookOpts, linkOpts: linkOpts };
        var template = utils_1.processData(info, data);
        var typeForModel = bookOpts.typeForModel, keyForAttr = bookOpts.keyForAttr;
        var typeForAttribute;
        if (typeof typeForModel === 'function') {
            typeForAttribute = typeForModel; // XXX remove `as ...` in typescript 2
        }
        else {
            // if the typeForModel object returns a falsy value, pluralize the attribute
            typeForAttribute = function (attr) { return typeForModel[attr] || inflection_1.pluralize(attr); };
        }
        // Override the template with the provided serializer options
        lodash_1.assign(template, { typeForAttribute: typeForAttribute, keyForAttribute: keyForAttr }, this.serialOpts);
        // Return the data in JSON API format
        var json = utils_1.toJSON(data);
        return new jsonapi_serializer_1.Serializer(type, template).serialize(json);
    };
    return Bookshelf;
}());
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = Bookshelf;
//# sourceMappingURL=index.js.map